"""
AB_Text.py
Developed for NISS AI Project by Kris Schnee.

Text manipulation code. Nothing glamorous here.
"""

##### INCLUDED MODULES #####

import string

##### MODULE ID #####

MODULE_NAME = "Aquablue: Text"
MODULE_NOTES = "Text-manipulation code."

##### STRING MANIPULATION #####


def StringSearch(source, query, return_start_only=False):
    """Returns first instance of string 'query' in 'source'. Returns x,x+y."""
    x = 0
    for x in range(0, len(source) - ( len(query) -1 )  ):
        failed = 0
        #print "Starting at x = " + str(x)
        for y in range( len(query) ):
            if failed == 0:
                #print "Checking x + y: " + str(x) + " + " + str(y)
                if source[x + y] != query[y]:
                    failed = 1
                    #print "Didn't match. (" + source[x + y] + " vs. " + query[y] + ")"
        if failed == 0:
            if return_start_only:
                return x
            else:
                return x, x+y
    ## If we're here, no match was found.
    if return_start_only:
        return -1
    else:
        return -1, -1

def ListSearch(source, query=[""]):
    """Search source list for any item in query list."""
    x = 0
    for x in range(len(source)):
        if source[x] in query:
            return x
    return -1

def ReplaceWord(source, target, change_to, return_list=True):
    if type(source) is str:
        source = string.split(source)
    target = target.lower()
    for n in range(len(source)):
        if source[n].lower() == target:
            source[n] = change_to
    if return_list:
        return source
    else:
        return string.join(source)

def StripPunctuation(source, taboo=",.!?\":;"):
    """Return a string with punctuation replaced by spaces."""
    ## The commented line works fine, but strips characters that are useful,
    ## like hyphens.
    for x in range(len(source)):
        #if source[x] in string.punctuation:
        if source[x] in taboo:
            source = source[:x] + " " + source[(x+1):]
    return source

def StripNewLines(source):
    while "\n" in source:
        x,y = StringSearch(source, "\n")
        source = source[:x] + " " + source[x+1:]
    return source

def StripArticles(source):
    """Return a string with articles deleted."""
    ## String must be chopped into words and rejoined.
    source = string.split(source)
    x = 0
    while x < len(source): ## Not a for-loop, since len(source) can change.
        if string.lower(source[x]) in ["a","an","the","some","that","those"]: ## OK, some of these aren't articles.
            source = source[:x] + source[(x+1):]
        x += 1
    return string.join(source)

def Strip(source,taboo):
    """Return a string with words in list "taboo" deleted."""
    ## Note: source checks each *word* for *words* in a taboo *list*.
    ## If you're trying to strip commas or something, use StripPunctuation
    ## with a second argument. Example:
    ## Strip("You can run with us.","with")
    ## (returns:) 'You can run us.'

    ## Chop the string into words and rejoin after scanning.
    source = string.split(source)
    x = 0
    while x < len(source): ## Not a for-loop, since len(source) can change.
        if string.lower(source[x]) in taboo:
            source = source[:x] + source[(x+1):]
        x += 1
    return string.join(source)

def ContainsPattern(source,pattern):
    """Look for a pattern of words in order within source."""
    ## Prep source by using StripPunctuation() first.
    ## Note that return value for a one-word pattern is 0 no matter what.
    ## But for one word, you could just say "pattern in source".
    source = string.split( string.lower(source) )
    pattern = string.split( string.lower(pattern) )
    x = 0
    for foo in pattern:
        if (foo in source[x:]) != 1: ## This word's not there at all.
            return 0
        else:
            for y in range(x, len(source) ): ## Search out its location. Not using source[x:] for speed(?).
                if source[y] == foo:
                    x = y ## Move the marker to make sure pattern's words are found in order.
                    break
    return x ## Success. Return which word ended the pattern.

def HideQuotedPhrases(text):
    """Disguise "some text in quotes" as ||a~~single~~word||."""
    done = False
    counter = 0
    while not done:
        counter += 1
        if counter > 5:
            print "Using escape mechanism."
            done = True ## Escape mechanism.
        openquote = string.find(text,'"')
        if openquote == -1: ## No more quotes.
            done = True
        elif openquote == len(text) -1:
            print "Stray closing quote; ignoring."
            done = True
        else:
            closequote = string.find(text[openquote+2:],'"')
            if closequote == -1: ## If no closing quote found (oops), set to -3 so it'll become -1, the end.
                closequote = -3
##                done = True
                quotedtext = "||"+text[openquote+1:].replace(" ","~~")+"||"
                text = text[:openquote] + quotedtext
            else:
                closequote += openquote
                quotedtext = "||"+text[openquote+1:closequote+2].replace(" ","~~")+"||"
                text = text[:openquote] + quotedtext + text[closequote+3:]
    return text

def RevealQuotedPhrases(text):
    """Undo HideQuotedPhrases."""
    done = False
    while not done:
        openquote = string.find(text,"||")
        if openquote == -1: ## No more quotes.
            done = True
        else:
            closequote = string.find(text[openquote+2:],"||")
            if closequote == -1: ## If no closing quote found (oops), set to -3 so it'll become -1, the end.
                closequote = -3
            closequote += openquote
            quotedtext = text[openquote:closequote+2].replace("~~"," ")
            quotedtext = quotedtext.replace("||","")
            text = text[:openquote] + '"' + quotedtext + '"' + text[closequote+4:]
    return text

def StripAndSplit(text):
    """Strip punctuation and split into words, treating "quoted stuff" as one word."""
    text = HideQuotedPhrases( text )
    quotes = []
    done = False
    while not done:
        openquote = text.find("||")
        if openquote == -1:
            done = True
        else:
            closequote = string.find(text[openquote+2:],"||")
            if closequote == -1: ## If no closing quote found (oops), set to -3 so it'll become -1, the end.
                closequote = -3
            closequote += openquote
            quotes.append(text[openquote:closequote+4])
            text = text[:openquote] + "~~~" + text[closequote+4:]
    text = StripPunctuation(text)
    for quote in quotes:
        quote_location = text.find("~~~")
        text = text[:quote_location] + quote + text[quote_location+3:]
    words = text.split()
    for x in range(len(words)):
        if words[x] and (words[x][0] == "|"):
            words[x] = RevealQuotedPhrases(words[x])
    return words


    sentence_cleaned = StripPunctuation( text )
    print sentence_cleaned
    words = string.split( sentence_cleaned )
    print words
    for x in range(len(words)):
        if words[x] and ("|" in words[x]):
            print "Changing to: " + str(RevealQuotedPhrases(words[x]))
            words[x] = RevealQuotedPhrases( words[x] )
    return text

def SentenceBreak(source):
    """Split string into a list of sentences."""
    if not source:
        return []
    if not (source[-1] in ".?!'\""):
        source += "."
    x = 0
    last_sentence_ended_at = 0
    sentences = []
    source = source.replace('.".','."')
    source = source.replace('?".','?"')
    source = source.replace('!".','!"')
    source_minus_one = len(source) - 1
    inside_a_quote = False
    while x < len(source):
        if source[x] == '"':
            if inside_a_quote:
                inside_a_quote = False
                if source[x-1] in ".?!":
                    sentences.append(source[last_sentence_ended_at:x+1])
                    last_sentence_ended_at = x+1
            else:
                inside_a_quote = True
        elif (source[x] in ".?!") and (not inside_a_quote):
            if (x < source_minus_one) and (source[x+1] == '"'):
                sentences.append(source[last_sentence_ended_at:x+2])
                last_sentence_ended_at = x+2
            else:
                sentences.append(source[last_sentence_ended_at:x+1])
                last_sentence_ended_at = x+1
        x += 1
    for x in range(len(sentences)):
        while sentences[x][0] == " ":
            sentences[x] = sentences[x][1:]
        while sentences[x][-1] == " ":
            sentences[x] = sentences[x][:-1]
    return sentences

def FancyList(list_of_words):
    result = ""
    if len(list_of_words) > 2:
        for word in list_of_words[:-2]:
            result += word + ", "
    if len(list_of_words) > 1:
        result += list_of_words[-2] + " and "
    result += list_of_words[-1]
    return result

## Not yet implemented.
def ApplyIRL(text="bear katamari in mind",filters=[{"*1":[0,0,2,3],"key":"bear *1 in mind","result":"remember *1"}]):
    """Apply "Idiom Reduction Language" filters, similar to those used in AIML.
    Filter example: ["bear *1 in mind","remember *1"] """
    if type(text) is list:
        words = text
    else:
        words = string.split(text)

    for filter in filters:
        filter_words = string.split(filter.get("key"))
        number_of_keys = string.count(filter,"*")
        for key in range(number_of_keys):
            numbers = filter.get("*"+str(key)) ## eg. "*1"
            if not numbers:
                print "Error: Key numbers for key "+str(key)+" don't exist."
                continue
            fore_start = numbers[0]
            fore_end = numbers[1]
            aft_start = numbers[2]

def ProcessIRLFilter(filter=["bear *a in mind","remember *a"]):
    """Turn a list filter like the above into a dictionary."""
    filter_words = string.split(filter)
    number_of_keys = string.count(filter,"*")

    for key in range(number_of_keys):
        pass



##### AUTORUN CODE #####
## Code that will auto-run if this file isn't/is included in a larger program.
if __name__ != "__main__":
##    print "Loaded module "+MODULE_NAME+"."
    pass
else:
    print "Running module "+MODULE_NAME+"."
    if MODULE_NOTES:
        print MODULE_NOTES
