import os
from ab_text import *

## Module ID
VERSION = "2005.2.24"
MODULE_NAME = "XML"
MODULE_COMMENTS = "A system for using the eXtensible Markup Language."


XML_DEBUG = False

def WriteHeader(file_object,top_level_tag_name=""):
    """Get an XML file off to a good start by making it browser-readable."""
    file_object.write("""<?xml version="1.0" encoding="ISO-8859-1"?>\n""")
    if top_level_tag_name:
        file_object.write("<"+top_level_tag_name+">")

def WriteCloser(file_object,top_level_tag_name=""):
    """For the sake of symmetry with WriteHeader."""
    file_object.write("</"+top_level_tag_name+">")

def GrabTag(text,target="",return_start=False,return_end=True,include_brackets=False):
    """Capture the next XML tag.
    Target should be without brackets, eg., to find </name>, search
    for target "/name". This lets you find tags with = in them."""
    if target: ## Search for a specific tag.
        start = StringSearch(text,"<"+target,True)
        if start != -1:
            start += 1
            end = StringSearch(text[start:],">",True)
            end += start
        else:
            end = -1
    else: ## No target; find any tag.
        start = StringSearch(text,"<",True)
        start += 1
        end = StringSearch(text[start:],">",True)
        end += start

    if start:
        if include_brackets:
            tag = text[start-1:end+1]
        else:
            tag = text[start:end]
    else:
        tag = ""
    if return_start and return_end:
        return tag, start, end
    elif return_start:
        return tag, start
    elif return_end:
        return tag, end
    else:
        return tag

def GrabTagPair(buffer,target,return_end=True):
    ## Note: returning an 'end' of 0 could cause problems. 'if GrabTagPair()...'
    tag, opening_tag_end = GrabTag(buffer, target)
    if tag:
        if tag == target:
            tag, closing_tag_start, closing_tag_end = GrabTag(buffer,"/"+target,True,True)
            if tag:
                if XML_DEBUG:
                    print "Found closing tag: " + tag
                result = buffer[opening_tag_end+1:closing_tag_start-1]
                if XML_DEBUG:
                    print "Content of tag pair: " + result
                if return_end:
                    return result, closing_tag_end
                else:
                    return result
    else:
        if return_end:
            return False, False
        else:
            return False


def GrabAllObjects(f,target,max=None):
    """Return all XML blocks of type 'target' from file 'f'."""
    f.seek(0) ## Move to start of file
    buffer = ""
    file_done = False
    results = []
    if (target[0] == "<") and (target[-1] == ">"):
        target = target[1:-1] ## Strip enclosing marks
    if XML_DEBUG:
        print "[GrabAllObjects] Starting hunt for type '"+target+"'."

    ## Find root tag, so we know where to stop loading.
    ## Assume the <?xml> tag and the root tag are on the first two lines,
    ## each alone, and that the real content doesn't begin till the third line.
    buffer += f.readline()
    tag = GrabTag(buffer,None,False,False)
    if tag and tag[:4] == "?xml":
        buffer = ""
        buffer += f.readline()
        tag = GrabTag(buffer,None,False,False)
        if tag:
            final_tag = "/" + tag ## This marks EOF.
            buffer = ""
            if XML_DEBUG:
                print "XML file is apparently valid."
                raw_input("Continue")
        else:
            return "Error: Malformed XML header."
    else:
        return "Error: Malformed XML header."

    while not final_tag in buffer:
        buffer += f.readline()
        tag = GrabTag(buffer, target, False, False)
        if tag:
            if XML_DEBUG:
                raw_input("Found target tag. Looking for match.")
            while not GrabTagPair(buffer, target):
                buffer += f.readline()
                if final_tag in buffer:
                    print "Error: Found file-ending tag before item-closing tag."
                    return

            ## Now we can get a complete item.
            grabbed_item, end = GrabTagPair(buffer, target)
            results.append(grabbed_item)
            if XML_DEBUG:
                print "--- Grabbed: " + grabbed_item
                raw_input("Grabbed complete item.")

            ## Have we reached a maximum number of results?
            if max and ( len(results) >= max ):
                return results
            
            ## Move ahead to the end of the tag pair.
            buffer = buffer[end:]

    return results
            

def GrabObjectTraits(input):
    """Chew up input and return its first-level tag contents."""
    results = []
    result_names = []
    while "<" in input:
        tag = GrabTag(input, None, False, False)
        if XML_DEBUG:
            print "Looking for tag '" + tag + "'"
        tagpair, end = GrabTagPair(input, tag)
        if tagpair:
            if XML_DEBUG:
                print "Appending."
            results.append(tagpair)
            result_names.append(tag)
        input = input[end+1:]
        if XML_DEBUG:
            print "New input: " + input
    return results, result_names

def Tag(type,content,newline=True):
    """Make a tag."""
    if content == "": ## Make blank tags for None, False, and 0.
        return ""
    x = "<"+type+">"+str(content)+"</"+type+">"
    if newline:
        x+= "\n"
    return x

class Foo:
    def __init__(self):
        self.name = "Foo"
        self.data = 42

class Bar:
    def __init__(self):
        self.name = "Bar"
        self.items = []
        for x in range(10):
            self.items.append( Foo() )
    def Write(self):
        result = ""
        for item in self.items:
            result.append("<name>"+item.name + "</name>\n<data>" + item.data + "</data>\n")
        return result

def WriteFile(object,filename="data.xml",header="<file>"):
    pass


## Autorun ##
if __name__ != "__main__":

##    print "Module '" + MODULE_NAME + "' loaded."
    pass

else:
    print "Running module '" + MODULE_NAME + "' alone.\n" + MODULE_COMMENTS

    x = "Junk<irr><sub>Foo</sub><pred>Bar</pred></irr><and>More</and>"
    print GrabObjectTraits(x)
