SETTINGS = {"island":{"desc":"An island.","down":"sand","up":"blue sky",
                      "intro":["You're on an island on a warm sunny day.","Below you is a beach of clean white sand, with a blue tropical sky above.","Salty wind blows from the sea as you wander, listening to the waves."],
                      "triggers":["water","seashell"]
                      }
            }

TRIGGERS = {"water":{"intro":["The water is cool and inviting.","As the water laps around you, something strange seems to be happening."],
                     "on_touch":["The water brushes against your *.","The water makes your skin tingle."],
                     },
            "seashell":{"intro":["You find a spiraling seashell that sparkles in the sunlight.","It's cool to the touch, and looks like it could be used as a horn.","You blow into the shell with a burst of air from your lungs, giving off a loud, echoing note.","You're dizzy from the exertion, or maybe from something else..."],
                        }
            }

CHANGE_FLAVOR_TEXT = {"texture":
                      {("skin","fur"):["The skin itches terribly.","You feel goosebumps, except that the faint hair that they raise is getting much denser!","Air currents ruffle the growing fur.","The new fur feels almost alive with sensation."],
                       ("fur","skin"):["The fur shrinks away, leaving behind bare skin.","The fur falls out in clumps.","As though in a dream the fur shimmers and fades."]
                       },
                      "shape":
                      {}
                      }

DEFAULT_INTERESTS = {"person":2,"dangerous":-9,"food":5,"shiny":3,"cute":3,"interesting":1}
DEFAULT_BELIEFS = {"nature:person":"person"}


TOUCH_EFFECT_CHANCE = 25
CHANGE_FLAVOR_TEXT_CHANCE = 25

import random
import copy
from ab_xml import *

TEMPLATE_DIRECTORY = "templates"


class CharBuilder:
    def __init__(self,**options):
        """Creates a description of a character's beliefs and body,
        based on XML templates."""
        self.templates = {}
        if options.get("load_templates",True):
            self.LoadCharTemplates()

    def LoadCharTemplates(self):
        file_list = os.listdir( os.path.join(os.getcwd(),TEMPLATE_DIRECTORY) )
        for filename in file_list:
            if filename[-3:].lower() != "xml":
                continue
            print "*** Loading file: "+filename+" ***"
            xml_file = file( os.path.join(TEMPLATE_DIRECTORY, filename) , "r" )
            templates_rawdata = GrabAllObjects(xml_file,"template")

            for data in templates_rawdata:
                traits, trait_names = GrabObjectTraits(data)
                if traits:
                    content = dict([(trait_names[n],traits[n]) for n in range(len(traits))])

                    template = {}

                    if content.has_key("name"):
                        template["name"] = content.get("name")
                    else:
                        template["name"] = "Unknown Template "+str(randint(0,999))

                    if content.has_key("nature"):
                        template["nature"] = content.get("nature")
                    else:
                        template["nature"] = ""

                    if content.has_key("beliefs"):
                        beliefs_data = eval(content["beliefs"])
                        beliefs = {}
                        for b in beliefs_data:
                            beliefs[ b ] = beliefs_data[ b ]
                        template["beliefs"] = beliefs

                    if content.has_key("parts"):
                        parts_data = eval(content["parts"])
                        parts = {}
                        for p in parts_data:
                            parts[ p ] = parts_data[ p ]
                        template["parts"] = parts

                    if content.has_key("covering"):
                        template["covering"] = eval(content["covering"])

##                    ## Debug: Report.
##                    print "\n--- Loaded template: "+template["name"]+" ("+template["nature"]+") ---"
##                    for key in template:
##                        if key not in ["name","nature"]:
##                            value = template[key]
##                            if type(value) in (str,int):
##                                print "-"+key+": "+str(value)
##                            else:
##                                print key+":"
##                                for innerkey in value:
##                                    print "    -"+innerkey+": "+str(value[innerkey])

                    ## We've extracted what info we can. Store this breed.
                    self.templates[ template["name"] ] = template

            xml_file.close()

    def Build(self,template_list=[],**options):
        """Create a character by overlaying templates.
        By inputting something like "male human", we get a description of
        the character's body and attitudes -- details like the color of
        their hair or whether they think of meat as food.

        If an attribute is given as a list, need to pick a random item from it.
        Eg., for hair color.

        Beliefs should be merged into a single beliefs set for the character,
        but the character should also keep separate copies so that we can
        notice interesting internal disputes: "As a hunter he knew the carcass
        to be food, but as a squirrel he found the idea revolting."

        Body parts should be merged into a single set.
        Where a part already exists, sizes are added (w/limits).
        """

        name = options.get("name","Grignr")
        data_only = options.get("data_only",True)

        ## Handle string input like "male fox".
        if type(template_list) is str:
            template_list = template_list.strip(",").split()

        print "Building character "+name+" "+str(template_list)+"."

        ## Always apply the default template first.
        template_list = ["default"] + template_list

        ## Begin building the character.
        char = {}
        desc = {}
        beliefs = {}
        parts = {}

        body_covering = {}

        ## For each template...
        for t in template_list:
            template = copy.deepcopy( self.templates.get( t ) )
            if not template:
                print "Template '"+t+"' not found."
                continue

            nature = template["nature"]
            desc[ nature ] = t ## Eg., stores that "breed" is "fox".

            print "Applying template: "+t+" (Nature = "+str(nature)+")"

            ## Handle beliefs.
            template_beliefs = template.get("beliefs")
            if template_beliefs:
                for key in template_beliefs:
                    current_value = beliefs.get(key,[])
                    this_template_value = template_beliefs[ key ]
                    if type(this_template_value) is str:
                        this_template_value = [this_template_value]
                    beliefs[ key ] = current_value + this_template_value

            ## Handle body covering, a set of defaults.
            ## The default covering for each body part is built via multiple
            ## templates, then applied after all templates are loaded.
            covering = template.get("covering",{})
            body_covering.update(covering)
##            print "Updating covering data to: "+str(body_covering)

            ## Handle parts. (Pun not intended.)
            ## Debug: Watch carefully to make sure old templates aren't overwritten.
            ## Bug: Numeric values aren't being summed yet.
            template_parts = template.get("parts")
            if template_parts:
                for key in template_parts:
                    current_part_data = copy.deepcopy(parts.get(key))
                    this_template_data = template_parts[ key ]
##                    print "Re: part: "+key
##                    print "Existing part data: "+str(current_part_data)
##                    print "This template data: "+str(this_template_data)
                    if current_part_data:
                        ## This part already exists.
##                        new_part_data = this_template_data ## Temp.
                        ## "update" is in-place, so we'll sort of copy c_p_d.
                        current_part_data.update( this_template_data )
                        new_part_data = current_part_data
                    else:
                        new_part_data = this_template_data
##                    print "New part data: "+str(new_part_data)

                    ## Replace list entries by picking a value.
                    ## Eg., pick an eye color from ["blue","grey" etc.].
                    for part_detail in new_part_data:
                        value = new_part_data[ part_detail ]
                        if type( value ) is list:
                            random_choice = value[ random.randint(0,len(value)-1) ]
                            new_part_data[ part_detail ] = random_choice
                            print "Chose '"+random_choice+"' for "+key+"."+part_detail+"."
                    parts[ key ] = new_part_data

        ## We've now applied all templates, so we know what parts exist and
        ## what they look like by default. Apply that data.
        ## Eg., each part of a fox is some fur color by default.

        ## First, specifify values if any are in list form.
##        print "Applying body covering: "+str(body_covering)
        for key in body_covering:
            value = body_covering[key]
            if type( value ) is list:
                random_choice = value[ random.randint(0,len(value)-1) ]
                body_covering[ key ] = random_choice
##                print "Specified that '"+key+"' is "+random_choice+"."
        for key in parts:
            part = parts[key]
            for default_trait in body_covering:
                if not part.has_key( default_trait ):
                    ## Put in the default.
                    parts[key][default_trait] = body_covering[default_trait]


        data = {"desc":desc,"beliefs":beliefs,"parts":parts}
        if data_only:
            return data
        char = Spark(name=name,data=data)
        return char







class Tiferet:
    def __init__(self,**options):
        pass
    def Start(self):
        setting_name = SETTINGS.keys()[random.randint(0,len(SETTINGS)-1)]
        setting = SETTINGS[setting_name]
        possible_triggers = setting.get("triggers")
        trigger_name = possible_triggers[random.randint(0,len(possible_triggers)-1)]
        trigger = TRIGGERS.get(trigger_name)
        print "Setting: "+setting_name.title()+". Trigger: "+trigger_name.title()+"."

        self.lines = []

        self.setting = setting
        self.trigger = trigger
        print "Ready to start."

    def DoChange(self):
        PARTS = ["arm","leg","face","hand"]
        part = PARTS[random.randint(0,3)]

        texture_change = ("skin","fur")

        ## Before
        touch_effects = self.trigger.get("on_touch")
        if touch_effects and random.randint(0,99) < TOUCH_EFFECT_CHANCE:
            effect = touch_effects[random.randint(0,len(touch_effects)-1)]
            effect = effect.replace("*",part)
            self.lines.append(effect)

        ## The change itself
        self.lines.append(part.title()+" has changed.")

        ## After
        if random.randint(0,99) < CHANGE_FLAVOR_TEXT_CHANCE:
            texture_effects = CHANGE_FLAVOR_TEXT["texture"].get(texture_change)
            if texture_effects:
                effect = texture_effects[random.randint(0,len(texture_effects)-1)]
            effect = effect.replace("*",part)
            self.lines.append(effect)

    def Go(self):
        self.Start()
        setting_intro = self.setting["intro"]
        trigger_intro = self.trigger["intro"]
        self.lines += setting_intro + trigger_intro

        change_count = 0
        while change_count < 3:
            change_count += 1
            self.DoChange()

        print "--- Starting! ---\n"
        for line in self.lines:
            print line


tf = Tiferet()
tf.Go()
##builder = CharBuilder()
##f = builder.Build("fox female")
